/*
 * MIT License
 *
 * Copyright (C) 2021-2023 by wangwenx190 (Yuhang Zhao)
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */

#pragma once

#include <FramelessHelper/Core/framelesshelpercore_global.h>
#include <optional>

FRAMELESSHELPER_BEGIN_NAMESPACE

class WindowBorderPainter;

class FRAMELESSHELPER_CORE_API WindowBorderPainterPrivate : public QObject
{
    Q_OBJECT
    Q_DECLARE_PUBLIC(WindowBorderPainter)
    Q_DISABLE_COPY_MOVE(WindowBorderPainterPrivate)

public:
    explicit WindowBorderPainterPrivate(WindowBorderPainter *q);
    ~WindowBorderPainterPrivate() override;

    Q_NODISCARD static WindowBorderPainterPrivate *get(WindowBorderPainter *q);
    Q_NODISCARD static const WindowBorderPainterPrivate *get(const WindowBorderPainter *q);

    Q_NODISCARD static int getNativeBorderThickness();
    Q_NODISCARD static QColor getNativeBorderColor(const bool active);
    Q_NODISCARD static Global::WindowEdges getNativeBorderEdges();

public Q_SLOTS:
    void paint(QPainter *painter, const QSize &size, const bool active) const;

private:
    void initialize();

private:
    WindowBorderPainter *q_ptr = nullptr;
    std::optional<int> m_thickness = std::nullopt;
    std::optional<Global::WindowEdges> m_edges = std::nullopt;
    std::optional<QColor> m_activeColor = std::nullopt;
    std::optional<QColor> m_inactiveColor = std::nullopt;
};

FRAMELESSHELPER_END_NAMESPACE
